from pydantic import BaseModel, Field
from typing import Optional, Union, Dict, List, Any
from apicase.common.enumeration import (
    BodyTypeEnum, ComparatorEnum
)


class RequestSchema(BaseModel):
    url: str = Field(..., description='请求路径')
    method: str = Field(..., description='请求方法')
    body_type: BodyTypeEnum = Field(BodyTypeEnum.NULL, description='请求体类型')
    data: Union[str, Dict, List, bytes] = Field(None, description='请求体')
    files: Dict = Field({}, description='上传的文件')
    path_params: Dict = Field({}, description='路径参数')
    query_params: Dict = Field({}, description='查询参数')
    headers: Dict = Field({}, description='请求头')
    cookies: Dict = Field({}, description='请求cookie')
    timeout: float = Field(None, description='请求超时时间')
    other: Dict = Field(None, description='其他')
    description: str = None


class JSONAssertSchema(BaseModel):
    expression: str = Field(...)
    comparator: ComparatorEnum
    value: Any = Field(...)



__version__ = '0.1.0'
__description__ = '接口自动化用例、测试的一站式解决方案。'


from apicase.schema import BaseModel
from apicase.client.router import APIRouter
from apicase.client.reuquest import session
from apicase.assertion import JSONAsserter
from apicase.common.enumeration import BodyTypeEnum

bt = BodyTypeEnum



from typing import Union

import jmespath


class APIResponse(object):
    response_json: Union[dict, list] = None

    def __init__(self, response):
        self.request = response.request
        self.status_code = response.status_code
        self.headers = response.headers
        self.json = response.json
        self.text = response.text

    def _init_json(self):
        if self.response_json is None:
            self.response_json = self.json()

    def jmespath(self, expression):
        self._init_json()
        return jmespath.search(expression=expression, data=self.response_json)



from pydantic import Field
from typing import Union, List, Any
from pydantic import BaseModel
import jmespath
from apicase.common.exception import JsonAssertError
from apicase.common.enumeration import ComparatorEnum


class JSONAssertSchema(BaseModel):
    expression: str = Field(...)
    comparator: ComparatorEnum
    value: Any = Field(...)


class JSONAsserter(object):
    assert_list = List[JSONAssertSchema]
    assert_results_list = List

    def __init__(self, a_list: list, equal_type: bool = True):
        self.a_list = a_list
        self.equal_type = equal_type
        self.init_data()

    def init_data(self):
        assert_list = []
        for i in self.a_list:
            assert_list.append(
                JSONAssertSchema(expression=i[0], comparator=i[1], value=i[2])
            )
        self.assert_list = assert_list

    def json_assert(self, response_json: Union[dict, list]):
        assert_results_list = []
        for i in self.assert_list:
            value = i.value
            comparator = i.comparator
            assert_value = jmespath.search(i.expression, response_json)
            if comparator is ComparatorEnum.EQUAL:
                assert_results_list.append([assert_value, assert_value == value, i])
            if comparator is ComparatorEnum.GREATER:
                assert_results_list.append([assert_value, assert_value > value, i])
            if comparator is ComparatorEnum.LESS:
                assert_results_list.append([assert_value, assert_value < value, i])
        self.assert_results_list = assert_results_list
        self._assert()

    def _assert(self):
        error_list = []
        for i in self.assert_results_list:
            if not i[1]:
                error_list.append(i)
        error_str = ''
        for i in error_list:
            assert_example = i[2]
            value = str(assert_example.value)
            assert_value = str(i[0])
            comparator = assert_example.comparator
            expression = assert_example.expression
            error_str = error_str + '\n\t JSON.{expression} {comparator} {value} \t' \
                                    'But: {assert_value} not {comparator} {value}'.format(expression=expression,
                                                                                          comparator=comparator.value,
                                                                                          value=value,
                                                                                          assert_value=assert_value,
                                                                                          )
        raise JsonAssertError('JSON 断言失败' + error_str)


class ResponseAsserter:
    pass


class CustomAsserter:
    pass



from typing import Union, Dict, List
import json

import requests
from pydantic import BaseModel

from apicase.client.response import APIResponse
from apicase.common.schema import RequestSchema
from apicase.common.enumeration import BodyTypeEnum

session = requests.Session()


class APIRequest(object):
    response = None

    def __init__(self, url: str, method: str, body_type: BodyTypeEnum, headers: dict = None, cookies: dict = None,
                 timeout: int = 30, pre: list = None, after: list = None, other: dict = None, description: str = None,
                 # -------------------------   参数分割   --------------------------------
                 path: str = None):
        self.path = path  # 提供该类属性，将接口路由展示报告上
        if pre is None:
            pre = list()
        if after is None:
            after = list()
        if headers is None:
            headers = list()
        if cookies is None:
            cookies = list()
        self.pre = pre
        self.after = after
        self.request = RequestSchema(
            url=url,
            method=method,
            body_type=body_type,
            headers=headers,
            cookies=cookies,
            timeout=timeout,
            other=other,
            description=description,
        )

    def send(self,
             schema: BaseModel = None,
             data: Union[str, Dict, List, bytes] = None,
             query_params: dict = None,
             path_params: dict = None,
             files: dict = None
             ):
        if not query_params:
            query_params = {}
        if not path_params:
            path_params = {}
        if not files:
            files = {}
        self.request.query_params = query_params
        self.request.path_params = path_params
        self.request.files = files
        self.request.data = data

        if schema:
            self.request.data = schema.dict()

        self.set_content_type()
        if self.pre:
            self.request = self.execution_processor(self.pre, self.request)
        response = session.request(
            method=self.request.method,
            url=self.request.url,
            params=self.request.query_params,
            data=self.request.data,
            files=self.request.files,
            headers=self.request.headers,
            cookies=self.request.cookies,
            timeout=self.request.timeout,
            verify=False  # 不认证证书
        )
        self.response = response
        response.json()
        if self.after:
            self.response = self.execution_processor(self.after, self.response)
        return APIResponse(self.response)

    @staticmethod
    def execution_processor(exec_list: list, parameter):
        for i in exec_list:
            parameter = i(parameter)
        return parameter

    def set_content_type(self):
        body_type = self.request.body_type
        content_type = None
        if not body_type:
            if body_type == BodyTypeEnum.JSON:
                content_type = 'application/json'
                self.request.data = json.dumps(self.request.data, ensure_ascii=False)
                self.request.data = self.request.data.encode('utf-8')
            elif body_type == BodyTypeEnum.TEXT:
                content_type = 'text/plain'
            elif body_type == BodyTypeEnum.XML:
                content_type = 'application/xml'
            elif body_type == BodyTypeEnum.FORM_URLENCODED:
                content_type = 'application/x-www-form-urlencoded'
            self.request.headers['Content-Type'] = content_type

    def add_pro(self, pro_list: list):
        pass

    def add_after(self, after_list: list):
        pass



from typing import NewType


class Case(object):
    pass


CaseType = NewType('CaseType', Case)


class Vars(object):
    __vars = {}

    def put(self, key, value) -> None:
        self.__vars[key] = value

    def get(self, key):
        return self.__vars.get(key)

    def delete_all(self):
        self.__vars = {}

    def get_all(self):
        return self.__vars

    # @staticmethod
    # def log(dict_path, value):
    #     data = '已经存入数据:' + 'dict_path【{}】'.format(str(dict_path)) + 'data【{}】'.format(str(value))
    #     # logger.debug(data)


VarsType = NewType('VarsType', Vars)



from enum import Enum


class BodyTypeEnum(Enum):
    NULL = None
    JSON = 'JSON'
    TEXT = 'TEXT'
    XML = 'XML'
    FORM_URLENCODED = 'x-www-form-urlencoded'
    FORM_DATA = 'form-data'


class ComparatorEnum(Enum):
    EQUAL = '='
    GREATER = '>'
    LESS = '<'



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['apicase', 'apicase.client', 'apicase.common', 'apicase.script']

package_data = \
{'': ['*']}

install_requires = \
['allure-pytest>=2.12.0,<3.0.0',
 'jmespath>=1.0.1,<2.0.0',
 'pydantic>=1.10.4,<2.0.0',
 'requests>=2.28.1,<3.0.0']

entry_points = \
{'console_scripts': ['apicase = apicase.script.cli:main']}

setup_kwargs = {
    'name': 'apicase',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 'guowenhe',
    'author_email': '18538570410@163.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from enum import Enum
from datetime import datetime
from typing import (
    List, Dict, Set, Tuple, Optional, Union
)
from pydantic import (
    BaseModel as Base, BaseSettings, PyObject, RedisDsn, PostgresDsn, AmqpDsn, Field,
)


def to_camel(string: str) -> str:
    """
    蟒蛇字符转换为小驼峰
    :param string:
    :return:
    """
    s = ''.join(word.capitalize() for word in string.split('_'))
    return s[0].lower() + s[1:]


class BaseModel(Base):
    class Config:
        arbitrary_types_allowed = True
        allow_population_by_field_name = False
        # alias_generator = to_camel  有bug，不能用
        smart_union = False
